@@ -16,11 +16,16 @@
 // Copyright (C) 2005-2007 Kristian Høgsberg <krh@redhat.com>
 // Copyright (C) 2006 Ed Catmur <ed@catmur.co.uk>
 // Copyright (C) 2007, 2008, 2011, 2013 Carlos Garcia Campos <carlosgc@gnome.org>
-// Copyright (C) 2007 Adrian Johnson <ajohnson@redneon.com>
-// Copyright (C) 2008, 2010 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2007, 2017 Adrian Johnson <ajohnson@redneon.com>
+// Copyright (C) 2008, 2010, 2015, 2016, 2018 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2010 Brian Ewins <brian.ewins@gmail.com>
-// Copyright (C) 2012, 2013, 2015 Jason Crain <jason@aquaticape.us>
+// Copyright (C) 2012, 2013, 2015, 2016 Jason Crain <jason@aquaticape.us>
 // Copyright (C) 2013 Thomas Freitag <Thomas.Freitag@alfa.de>
+// Copyright (C) 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by the LiMux project of the city of Munich
+// Copyright (C) 2018 Sanchit Anand <sanxchit@gmail.com>
+// Copyright (C) 2018 Nelson Benítez León <nbenitezl@gmail.com>
+// Copyright (C) 2019 Oliver Sander <oliver.sander@tu-dresden.de>
+// Copyright (C) 2019 Dan Shea <dan.shea@logical-innovations.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -30,19 +35,13 @@
 #ifndef TEXTOUTPUTDEV_H
 #define TEXTOUTPUTDEV_H
 
-#ifdef USE_GCC_PRAGMAS
-#pragma interface
-#endif
-
 #include "poppler-config.h"
 #include <stdio.h>
-#include "goo/gtypes.h"
 #include "GfxFont.h"
 #include "GfxState.h"
 #include "OutputDev.h"
 
 class GooString;
-class GooList;
 class Gfx;
 class GfxFont;
 class GfxState;
@@ -55,6 +54,8 @@ class TextLine;
 class TextLineFrag;
 class TextBlock;
 class TextFlow;
+class TextLink;
+class TextUnderline;
 class TextWordList;
 class TextPage;
 class TextSelectionVisitor;
@@ -79,34 +80,37 @@ public:
   TextFontInfo(GfxState *state);
   ~TextFontInfo();
 
-  GBool matches(GfxState *state);
-  GBool matches(TextFontInfo *fontInfo);
+  TextFontInfo(const TextFontInfo &) = delete;
+  TextFontInfo& operator=(const TextFontInfo &) = delete;
+
+  bool matches(GfxState *state) const;
+  bool matches(const TextFontInfo *fontInfo) const;
 
   // Get the font ascent, or a default value if the font is not set
-  double getAscent();
+  double getAscent() const;
 
   // Get the font descent, or a default value if the font is not set
-  double getDescent();
+  double getDescent() const;
 
   // Get the writing mode (0 or 1), or 0 if the font is not set
-  int getWMode();
+  int getWMode() const;
 
-#if TEXTOUT_WORD_LIST
+#ifdef TEXTOUT_WORD_LIST
   // Get the font name (which may be NULL).
-  GooString *getFontName() { return fontName; }
+  const GooString *getFontName() const { return fontName; }
 
   // Get font descriptor flags.
-  GBool isFixedWidth() { return flags & fontFixedWidth; }
-  GBool isSerif() { return flags & fontSerif; }
-  GBool isSymbolic() { return flags & fontSymbolic; }
-  GBool isItalic() { return flags & fontItalic; }
-  GBool isBold() { return flags & fontBold; }
+  bool isFixedWidth() const { return flags & fontFixedWidth; }
+  bool isSerif() const { return flags & fontSerif; }
+  bool isSymbolic() const { return flags & fontSymbolic; }
+  bool isItalic() const { return flags & fontItalic; }
+  bool isBold() const { return flags & fontBold; }
 #endif
 
 private:
 
   GfxFont *gfxFont;
-#if TEXTOUT_WORD_LIST
+#ifdef TEXTOUT_WORD_LIST
   GooString *fontName;
   int flags;
 #endif
@@ -124,23 +128,26 @@ class TextWord {
 public:
 
   // Constructor.
-  TextWord(GfxState *state, int rotA, double fontSize);
+  TextWord(const GfxState *state, int rotA, double fontSize);
 
   // Destructor.
   ~TextWord();
 
+  TextWord(const TextWord &) = delete;
+  TextWord& operator=(const TextWord &) = delete;
+
   // Add a character to the word.
   void addChar(GfxState *state, TextFontInfo *fontA, double x, double y,
 	       double dx, double dy, int charPosA, int charLen,
-	       CharCode c, Unicode u, Matrix textMatA);
+	       CharCode c, Unicode u, const Matrix &textMatA);
 
   // Attempt to add a character to the word as a combining character.
   // Either character u or the last character in the word must be an
   // acute, dieresis, or other combining character.  Returns true if
   // the character was added.
-  GBool addCombining(GfxState *state, TextFontInfo *fontA, double fontSizeA, double x, double y,
+  bool addCombining(GfxState *state, TextFontInfo *fontA, double fontSizeA, double x, double y,
 		     double dx, double dy, int charPosA, int charLen,
-		     CharCode c, Unicode u, Matrix textMatA);
+		     CharCode c, Unicode u, const Matrix &textMatA);
 
   // Merge <word> onto the end of <this>.
   void merge(TextWord *word);
@@ -165,7 +172,7 @@ public:
   // Get the next TextWord on the linked list.
   TextWord *getNext() { return next; }
 
-#if TEXTOUT_WORD_LIST
+#ifdef TEXTOUT_WORD_LIST
   int getLength() { return len; }
   const Unicode *getChar(int idx) { return &text[idx]; }
   GooString *getText();
@@ -180,13 +187,13 @@ public:
   int getRotation() { return rot; }
   int getCharPos() { return charPos[0]; }
   int getCharLen() { return charPos[len] - charPos[0]; }
-  GBool getSpaceAfter() { return spaceAfter; }
+  bool getSpaceAfter() { return spaceAfter; }
 #endif
-  GBool isUnderlined() { return underlined; }
+  bool isUnderlined() { return underlined; }
   AnnotLink *getLink() { return link; }
   double getEdge(int i) { return edge[i]; }
   double getBaseline () { return base; }
-  GBool hasSpaceAfter  () { return spaceAfter; }
+  bool hasSpaceAfter  () { return spaceAfter; }
   TextWord* nextWord () { return next; };
 private:
   void ensureCapacity(int capacity);
@@ -194,6 +201,7 @@ private:
 
   int rot;			// rotation, multiple of 90 degrees
 				//   (0, 1, 2, or 3)
+  int wMode;			// horizontal (0) or vertical (1) writing mode
   double xMin, xMax;		// bounding box x coordinates
   double yMin, yMax;		// bounding box y coordinates
   double base;			// baseline x or y coordinate
@@ -209,18 +217,17 @@ private:
   TextFontInfo **font;		// font information for each char
   Matrix *textMat;		// transformation matrix for each char
   double fontSize;		// font size
-  GBool spaceAfter;		// set if there is a space between this
+  bool spaceAfter;		// set if there is a space between this
 				//   word and the next word on the line
+  bool underlined;
   TextWord *next;		// next word in line
-  int wMode;			// horizontal (0) or vertical (1) writing mode
 
-#if TEXTOUT_WORD_LIST
+#ifdef TEXTOUT_WORD_LIST
   double colorR,		// word color
          colorG,
          colorB;
 #endif
 
-  GBool underlined;
   AnnotLink *link;
 
   friend class TextPool;
@@ -244,6 +251,9 @@ public:
   TextPool();
   ~TextPool();
 
+  TextPool(const TextPool &) = delete;
+  TextPool& operator=(const TextPool &) = delete;
+
   TextWord *getPool(int baseIdx) { return pool[baseIdx - minBaseIdx]; }
   void setPool(int baseIdx, TextWord *p) { pool[baseIdx - minBaseIdx] = p; }
 
@@ -276,6 +286,9 @@ public:
   TextLine(TextBlock *blkA, int rotA, double baseA);
   ~TextLine();
 
+  TextLine(const TextLine &) = delete;
+  TextLine& operator=(const TextLine &) = delete;
+
   void addWord(TextWord *word);
 
   // Return the distance along the primary axis between <this> and
@@ -308,7 +321,7 @@ public:
   TextLine *getNext() { return next; }
 
   // Returns true if the last char of the line is a hyphen.
-  GBool isHyphenated() { return hyphenated; }
+  bool isHyphenated() { return hyphenated; }
 
 private:
 
@@ -326,11 +339,14 @@ private:
   int *col;			// starting column number of each Unicode char
   int len;			// number of Unicode chars
   int convertedLen;		// total number of converted characters
-  GBool hyphenated;		// set if last char is a hyphen
+  bool hyphenated;		// set if last char is a hyphen
   TextLine *next;		// next line in block
   Unicode *normalized;		// normalized form of Unicode text
   int normalized_len;		// number of normalized Unicode chars
   int *normalized_idx;		// indices of normalized chars into Unicode text
+  Unicode *ascii_translation;	// ascii translation from the normalized text
+  int ascii_len;		// length of ascii translation text
+  int *ascii_idx;		// indices of ascii chars into Unicode text of line
 
   friend class TextLineFrag;
   friend class TextBlock;
@@ -353,6 +369,9 @@ public:
   TextBlock(TextPage *pageA, int rotA);
   ~TextBlock();
 
+  TextBlock(const TextBlock &) = delete;
+  TextBlock& operator=(const TextBlock &) = delete;
+
   void addWord(TextWord *word);
 
   void coalesce(UnicodeMap *uMap, double fixedPitch);
@@ -370,7 +389,7 @@ public:
 
   // Returns true if <this> is below <blk>, relative to the page's
   // primary rotation.
-  GBool isBelow(TextBlock *blk);
+  bool isBelow(TextBlock *blk);
 
   void visitSelection(TextSelectionVisitor *visitor,
 		      PDFRectangle *selection,
@@ -389,13 +408,17 @@ public:
 
 private:
 
-  GBool isBeforeByRule1(TextBlock *blk1);
-  GBool isBeforeByRepeatedRule1(TextBlock *blkList, TextBlock *blk1);
-  GBool isBeforeByRule2(TextBlock *blk1);
+  bool isBeforeByRule1(TextBlock *blk1);
+  bool isBeforeByRepeatedRule1(TextBlock *blkList, TextBlock *blk1);
+  bool isBeforeByRule2(TextBlock *blk1);
 
   int visitDepthFirst(TextBlock *blkList, int pos1,
 		      TextBlock **sorted, int sortPos,
-		      GBool* visited);
+		      bool* visited);
+  int visitDepthFirst(TextBlock *blkList, int pos1,
+		      TextBlock **sorted, int sortPos,
+		      bool* visited,
+		      TextBlock **cache, int cacheSize);
 
   TextPage *page;		// the parent page
   int rot;			// text rotation
@@ -405,7 +428,7 @@ private:
   double ExMin, ExMax;		// extended bounding box x coordinates
   double EyMin, EyMax;		// extended bounding box y coordinates
   int tableId;			// id of table to which this block belongs
-  GBool tableEnd;		// is this block at end of line of actual table
+  bool tableEnd;		// is this block at end of line of actual table
 
   TextPool *pool;		// pool of words (used only until lines
 				//   are built)
@@ -438,6 +461,9 @@ public:
   TextFlow(TextPage *pageA, TextBlock *blk);
   ~TextFlow();
 
+  TextFlow(const TextFlow &) = delete;
+  TextFlow& operator=(const TextFlow &) = delete;
+
   // Add a block to the end of this flow.
   void addBlock(TextBlock *blk);
 
@@ -445,7 +471,7 @@ public:
   // it uses a font no larger than the last block added to the flow,
   // and (2) it fits within the flow's [priMin, priMax] along the
   // primary axis.
-  GBool blockFits(TextBlock *blk, TextBlock *prevBlk);
+  bool blockFits(TextBlock *blk, TextBlock *prevBlk);
 
   // Get the head of the linked list of TextBlocks.
   TextBlock *getBlocks() { return blocks; }
@@ -467,7 +493,7 @@ private:
   friend class TextPage;
 };
 
-#if TEXTOUT_WORD_LIST
+#ifdef TEXTOUT_WORD_LIST
 
 //------------------------------------------------------------------------
 // TextWordList
@@ -480,10 +506,13 @@ public:
   // text->rawOrder is true), physical layout order (if <physLayout>
   // is true and text->rawOrder is false), or reading order (if both
   // flags are false).
-  TextWordList(TextPage *text, GBool physLayout);
+  TextWordList(TextPage *text, bool physLayout);
 
   ~TextWordList();
 
+  TextWordList(const TextWordList &) = delete;
+  TextWordList& operator=(const TextWordList &) = delete;
+
   // Return the number of words on the list.
   int getLength();
 
@@ -492,15 +521,15 @@ public:
 
 private:
 
-  GooList *words;			// [TextWord]
+  std::vector<TextWord*> *words;
 };
 
 #endif // TEXTOUT_WORD_LIST
 
 class TextWordSelection {
 public:
-  TextWordSelection(TextWord *word, int begin, int end)
-    : word(word), begin(begin), end(end)
+  TextWordSelection(TextWord *wordA, int beginA, int endA)
+    : word(wordA), begin(beginA), end(endA)
   {
   }
 
@@ -525,7 +554,10 @@ class TextPage {
 public:
 
   // Constructor.
-  TextPage(GBool rawOrderA);
+  TextPage(bool rawOrderA, bool discardDiagA = false);
+
+  TextPage(const TextPage &) = delete;
+  TextPage& operator=(const TextPage &) = delete;
 
   void incRefCnt();
   void decRefCnt();
@@ -563,7 +595,7 @@ public:
   void addLink(int xMin, int yMin, int xMax, int yMax, AnnotLink *link);
 
   // Coalesce strings that look like parts of the same line.
-  void coalesce(GBool physLayout, double fixedPitch, GBool doHTML);
+  void coalesce(bool physLayout, double fixedPitch, bool doHTML);
 
   // Find a string.  If <startAtTop> is true, starts looking at the
   // top of the page; else if <startAtLast> is true, starts looking
@@ -572,11 +604,23 @@ public:
   // bottom of the page; else if <stopAtLast> is true, stops looking
   // just before the last find result; else stops looking at
   // <xMax>,<yMax>.
-  GBool findText(Unicode *s, int len,
-		 GBool startAtTop, GBool stopAtBottom,
-		 GBool startAtLast, GBool stopAtLast,
-		 GBool caseSensitive, GBool backward,
-		 GBool wholeWord,
+  bool findText(Unicode *s, int len,
+		 bool startAtTop, bool stopAtBottom,
+		 bool startAtLast, bool stopAtLast,
+		 bool caseSensitive, bool backward,
+		 bool wholeWord,
+		 double *xMin, double *yMin,
+		 double *xMax, double *yMax);
+
+  // Adds new parameter ignoreDiacritics, which will do diacritics
+  // insensitive search, i.e. ignore accents, umlauts, diaeresis,etc.
+  // while matching. This option will be ignored if <s> contains characters
+  // which are not pure ascii.
+  bool findText(Unicode *s, int len,
+		 bool startAtTop, bool stopAtBottom,
+		 bool startAtLast, bool stopAtLast,
+		 bool caseSensitive, bool ignoreDiacritics,
+		 bool backward, bool wholeWord,
 		 double *xMin, double *yMin,
 		 double *xMax, double *yMax);
 
@@ -595,41 +639,41 @@ public:
 		     SelectionStyle style,
 		     GfxColor *glyph_color, GfxColor *box_color);
 
-  GooList *getSelectionRegion(PDFRectangle *selection,
+  std::vector<PDFRectangle*> *getSelectionRegion(PDFRectangle *selection,
 			      SelectionStyle style,
 			      double scale);
 
   GooString *getSelectionText(PDFRectangle *selection,
 			      SelectionStyle style);
 
-  GooList **getSelectionWords(PDFRectangle *selection,
+  std::vector<TextWordSelection*> **getSelectionWords(PDFRectangle *selection,
                               SelectionStyle style,
                               int *nLines);
 
   // Find a string by character position and length.  If found, sets
   // the text bounding rectangle and returns true; otherwise returns
   // false.
-  GBool findCharRange(int pos, int length,
+  bool findCharRange(int pos, int length,
 		      double *xMin, double *yMin,
 		      double *xMax, double *yMax);
 
   // Dump contents of page to a file.
   void dump(void *outputStream, TextOutputFunc outputFunc,
-	    GBool physLayout);
+	    bool physLayout);
 
   // Get the head of the linked list of TextFlows.
   TextFlow *getFlows() { return flows; }
 
   // If true, will combine characters when a base and combining
   // character are drawn on eachother.
-  void setMergeCombining(GBool merge);
+  void setMergeCombining(bool merge);
 
-#if TEXTOUT_WORD_LIST
+#ifdef TEXTOUT_WORD_LIST
   // Build a flat word list, in content stream order (if
   // this->rawOrder is true), physical layout order (if <physLayout>
   // is true and this->rawOrder is false), or reading order (if both
   // flags are false).
-  TextWordList *makeWordList(GBool physLayout);
+  TextWordList *makeWordList(bool physLayout);
 #endif
 
 private:
@@ -638,11 +682,12 @@ private:
   ~TextPage();
   
   void clear();
-  void assignColumns(TextLineFrag *frags, int nFrags, GBool rot);
+  void assignColumns(TextLineFrag *frags, int nFrags, bool rot);
   int dumpFragment(Unicode *text, int len, UnicodeMap *uMap, GooString *s);
 
-  GBool rawOrder;		// keep text in content stream order
-  GBool mergeCombining;		// merge when combining and base characters
+  bool rawOrder;		// keep text in content stream order
+  bool discardDiag;		// discard diagonal text
+  bool mergeCombining;		// merge when combining and base characters
 				// are drawn on top of each other
 
   double pageWidth, pageHeight;	// width and height of current page
@@ -653,29 +698,29 @@ private:
   double curFontSize;		// current font size
   int nest;			// current nesting level (for Type 3 fonts)
   int nTinyChars;		// number of "tiny" chars seen so far
-  GBool lastCharOverlap;	// set if the last added char overlapped the
+  bool lastCharOverlap;	// set if the last added char overlapped the
 				//   previous char
+  bool diagonal;		// whether the current text is diagonal
 
   TextPool *pools[4];		// a "pool" of TextWords for each rotation
   TextFlow *flows;		// linked list of flows
   TextBlock **blocks;		// array of blocks, in yx order
   int nBlocks;			// number of blocks
   int primaryRot;		// primary rotation
-  GBool primaryLR;		// primary direction (true means L-to-R,
+  bool primaryLR;		// primary direction (true means L-to-R,
 				//   false means R-to-L)
   TextWord *rawWords;		// list of words, in raw order (only if
 				//   rawOrder is set)
   TextWord *rawLastWord;	// last word on rawWords list
 
-  GooList *fonts;			// all font info objects used on this
-				//   page [TextFontInfo]
+  std::vector<TextFontInfo*> *fonts;// all font info objects used on this page
 
   double lastFindXMin,		// coordinates of the last "find" result
          lastFindYMin;
-  GBool haveLastFind;
+  bool haveLastFind;
 
-  GooList *underlines;		// [TextUnderline]
-  GooList *links;		// [TextLink]
+  std::vector<TextUnderline*> *underlines;
+  std::vector<TextLink*> *links;
 
   int refCnt;
 
@@ -698,10 +743,13 @@ public:
   ActualText(TextPage *out);
   ~ActualText();
 
+  ActualText(const ActualText &) = delete;
+  ActualText& operator=(const ActualText &) = delete;
+
   void addChar(GfxState *state, double x, double y,
 	       double dx, double dy,
 	       CharCode c, int nBytes, Unicode *u, int uLen);
-  void begin(GfxState *state, GooString *text);
+  void begin(GfxState *state, const GooString *text);
   void end(GfxState *state);
 
 private:
@@ -727,77 +775,79 @@ public:
   // written (this is useful, e.g., for searching text).  If
   // <physLayoutA> is true, the original physical layout of the text
   // is maintained.  If <rawOrder> is true, the text is kept in
-  // content stream order.
-  TextOutputDev(char *fileName, GBool physLayoutA,
-		double fixedPitchA, GBool rawOrderA,
-		GBool append);
+  // content stream order.  If <discardDiag> is true, diagonal text
+  // is removed from output.
+  TextOutputDev(const char *fileName, bool physLayoutA,
+		double fixedPitchA, bool rawOrderA,
+		bool append, bool discardDiagA = false);
 
   // Create a TextOutputDev which will write to a generic stream.  If
   // <physLayoutA> is true, the original physical layout of the text
   // is maintained.  If <rawOrder> is true, the text is kept in
-  // content stream order.
+  // content stream order.  If <discardDiag> is true, diagonal text
+  // is removed from output.
   TextOutputDev(TextOutputFunc func, void *stream,
-		GBool physLayoutA, double fixedPitchA,
-		GBool rawOrderA);
+		bool physLayoutA, double fixedPitchA,
+		bool rawOrderA, bool discardDiagA = false);
 
   // Destructor.
-  virtual ~TextOutputDev();
+  ~TextOutputDev();
 
   // Check if file was successfully created.
-  virtual GBool isOk() { return ok; }
+  virtual bool isOk() { return ok; }
 
   //---- get info about output device
 
   // Does this device use upside-down coordinates?
   // (Upside-down means (0,0) is the top left corner of the page.)
-  virtual GBool upsideDown() { return gTrue; }
+  bool upsideDown() override { return true; }
 
   // Does this device use drawChar() or drawString()?
-  virtual GBool useDrawChar() { return gTrue; }
+  bool useDrawChar() override { return true; }
 
   // Does this device use beginType3Char/endType3Char?  Otherwise,
   // text in Type 3 fonts will be drawn with drawChar/drawString.
-  virtual GBool interpretType3Chars() { return gFalse; }
+  bool interpretType3Chars() override { return false; }
 
   // Does this device need non-text content?
-  virtual GBool needNonText() { return gFalse; }
+  bool needNonText() override { return false; }
 
   // Does this device require incCharCount to be called for text on
   // non-shown layers?
-  virtual GBool needCharCount() { return gTrue; }
+  bool needCharCount() override { return true; }
 
   //----- initialization and control
 
   // Start a page.
-  virtual void startPage(int pageNum, GfxState *state, XRef *xref);
+  void startPage(int pageNum, GfxState *state, XRef *xref) override;
 
   // End a page.
-  virtual void endPage();
+  void endPage() override;
 
   //----- save/restore graphics state
-  virtual void restoreState(GfxState *state);
+  void restoreState(GfxState *state) override;
 
   //----- update text state
-  virtual void updateFont(GfxState *state);
+  void updateFont(GfxState *state) override;
 
   //----- text drawing
-  virtual void beginString(GfxState *state, GooString *s);
-  virtual void endString(GfxState *state);
-  virtual void drawChar(GfxState *state, double x, double y,
-			double dx, double dy,
-			double originX, double originY,
-			CharCode c, int nBytes, Unicode *u, int uLen);
-  virtual void incCharCount(int nChars);
-  virtual void beginActualText(GfxState *state, GooString *text);
-  virtual void endActualText(GfxState *state);
+  void beginString(GfxState *state, const GooString *s) override;
+  void endString(GfxState *state) override;
+  void drawChar(GfxState *state, double x, double y,
+		double dx, double dy,
+		double originX, double originY,
+		CharCode c, int nBytes, Unicode *u, int uLen) override;
+  void incCharCount(int nChars) override;
+  void beginActualText(GfxState *state, const GooString *text) override;
+  void endActualText(GfxState *state) override;
 
   //----- path painting
-  virtual void stroke(GfxState *state);
-  virtual void fill(GfxState *state);
-  virtual void eoFill(GfxState *state);
+  void stroke(GfxState *state) override;
+  void fill(GfxState *state) override;
+  void eoFill(GfxState *state) override;
 
   //----- link borders
-  virtual void processLink(AnnotLink *link);
+  void processLink(AnnotLink *link) override;
 
   //----- special access
 
@@ -808,11 +858,11 @@ public:
   // bottom of the page; else if <stopAtLast> is true, stops looking
   // just before the last find result; else stops looking at
   // <xMax>,<yMax>.
-  GBool findText(Unicode *s, int len,
-		 GBool startAtTop, GBool stopAtBottom,
-		 GBool startAtLast, GBool stopAtLast,
-		 GBool caseSensitive, GBool backward,
-		 GBool wholeWord,
+  bool findText(Unicode *s, int len,
+		 bool startAtTop, bool stopAtBottom,
+		 bool startAtLast, bool stopAtLast,
+		 bool caseSensitive, bool backward,
+		 bool wholeWord,
 		 double *xMin, double *yMin,
 		 double *xMax, double *yMax);
 
@@ -823,7 +873,7 @@ public:
   // Find a string by character position and length.  If found, sets
   // the text bounding rectangle and returns true; otherwise returns
   // false.
-  GBool findCharRange(int pos, int length,
+  bool findCharRange(int pos, int length,
 		      double *xMin, double *yMin,
 		      double *xMax, double *yMax);
 
@@ -832,7 +882,7 @@ public:
 		     SelectionStyle style,
 		     GfxColor *glyph_color, GfxColor *box_color);
 
-  GooList *getSelectionRegion(PDFRectangle *selection,
+  std::vector<PDFRectangle*> *getSelectionRegion(PDFRectangle *selection,
 			      SelectionStyle style,
 			      double scale);
 
@@ -841,9 +891,9 @@ public:
 
   // If true, will combine characters when a base and combining
   // character are drawn on eachother.
-  void setMergeCombining(GBool merge);
+  void setMergeCombining(bool merge);
 
-#if TEXTOUT_WORD_LIST
+#ifdef TEXTOUT_WORD_LIST
   // Build a flat word list, in content stream order (if
   // this->rawOrder is true), physical layout order (if
   // this->physLayout is true and this->rawOrder is false), or reading
@@ -856,23 +906,30 @@ public:
   TextPage *takeText();
 
   // Turn extra processing for HTML conversion on or off.
-  void enableHTMLExtras(GBool doHTMLA) { doHTML = doHTMLA; }
+  void enableHTMLExtras(bool doHTMLA) { doHTML = doHTMLA; }
+
+  // Get the head of the linked list of TextFlows for the
+  // last rasterized page.
+  TextFlow *getFlows();
 
 private:
 
   TextOutputFunc outputFunc;	// output function
   void *outputStream;		// output stream
-  GBool needClose;		// need to close the output file?
+  bool needClose;		// need to close the output file?
 				//   (only if outputStream is a FILE*)
   TextPage *text;		// text for the current page
-  GBool physLayout;		// maintain original physical layout when
+  bool physLayout;		// maintain original physical layout when
 				//   dumping text
   double fixedPitch;		// if physLayout is true and this is non-zero,
 				//   assume fixed-pitch characters with this
 				//   width
-  GBool rawOrder;		// keep text in content stream order
-  GBool doHTML;			// extra processing for HTML conversion
-  GBool ok;			// set up ok?
+  bool rawOrder;		// keep text in content stream order
+  bool discardDiag;     // Diagonal text, i.e., text that is not close to one of the
+				//0, 90, 180, or 270 degree axes, is discarded. This is useful
+				// to skip watermarks drawn on top of body text, etc.
+  bool doHTML;			// extra processing for HTML conversion
+  bool ok;			// set up ok?
 
   ActualText *actualText;
 };
